--[[--------------------------------------------------------------------------------
  ItemSyncCore BagView Framework

  Author:  Derkyle
  Website: http://www.manaflux.com
-----------------------------------------------------------------------------------]]


ISync_BV_Show_Stacked 	= 0;	--show stacked items in inventory window
ISync_BV_Show_Nill 	= 0;	--show empty value prices in inventory window
ISync_BV_Sort_Value 	= 0;	--sort by value in inventory window
ISync_BV_Sort_Rarity 	= 0;	--sort by rarity in inventory window

local ISync_BVList 	= { };	--stores the information of each item in an array




---------------------------------------------------
-- ISync:BV_Load
---------------------------------------------------
function ISync:BV_Load()

	---------------------------------------------------
	-- Register our slash command
	SLASH_ISYNCBAGVIEW1 = "/bagview";
	SLASH_ISYNCBAGVIEW2 = "/bv";
	SLASH_ISYNCBAGVIEW3 = "/bagv";
	SlashCmdList["ISYNCBAGVIEW"] = function(msg)

		if(ISyncVarLoad == 1) then


			if ( ISync_BV_Frame:IsVisible() ) then 
				ISync_BV_Frame:Hide();
			else
				ISync_BV_Frame:Show();
			end


		end
	
	end
	
	--add our frames to the special frames, this way when a user press's escape it will
	--close the window
	tinsert(UISpecialFrames, "ISync_BV_Frame");
    
	    
end


---------------------------------------------------
-- ISync:BV_Binding
---------------------------------------------------
function ISync:BV_Binding()

    if ISync_BV_Frame:IsVisible() then
	HideUIPanel(ISync_BV_Frame);
    else
	ShowUIPanel(ISync_BV_Frame);
    end
    
	    
end


---------------------------------------------------
-- ISync:BV_Refresh()
---------------------------------------------------
function ISync:BV_Refresh()

	ISync_BVList = nil; --reset
	
	local offset = FauxScrollFrame_GetOffset(ISync_IC_ScrollFrame);
	
	if( offset ) then
		--set to last offset
		FauxScrollFrame_SetOffset(ISync_IC_ScrollFrame, offset);	
	else
		--set to zero offset
		FauxScrollFrame_SetOffset(ISync_IC_ScrollFrame, 0);
	end
	
	ISync:BV_BuildIndex();
	ISync:BV_UpdateScrollFrame();

end


---------------------------------------------------
-- ISync:BV_BuildIndex()
---------------------------------------------------
function ISync:BV_BuildIndex()

	--make sure we have something to work with
	if(not ISyncDB) then return nil; end
	if(not ISync_RealmNum) then return nil; end
	if(not ISyncDB[ISync_RealmNum]) then return nil; end
		
	--reset inven array
	ISync_BVList =  nil;
	ISync_BVList = { };

	local iNew = 1;
    	local costOfItem, sColor, sParseLink;
    
    	--reset values
    	if(ISync_BV_Show_Stacked == nil) then ISync_BV_Show_Stacked = 0; end
    	if(ISync_BV_Show_Nill == nil) then ISync_BV_Show_Nill = 0; end
    	
    	--do the loops through all the bags
	for bag=0,NUM_BAG_FRAMES do
		for slot=1,GetContainerNumSlots(bag) do

		    local link = GetContainerItemLink(bag, slot);

		    if (link) then

			--grab the name and itemID
			local name = ISync:NameFromLink(link);
			
			if(name and ISync:GetData(name, "info") and ISync:GetData(name, "item")) then
			
				--grab the item information from the bag, slot
				local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
				
				--we have to have an item count
				if(itemCount) then

					costOfItem = 0;
					
					--get price
					sParseLink = nil; --reset
					sParseLink = string.gsub(ISync:GetData(name, "info"), "^(.-):(.-):(.-):(.-):(.-):(.-):(.-):(.-):(.-):(.-)$", "%2");


					--get the cost of the item if there is one
					if(ISyncDB[ISync_RealmNum][name] and sParseLink and sParseLink ~= "") then
						costOfItem = tonumber(sParseLink);
					end

					--calculate the stacked price of the item
					if (costOfItem and costOfItem > 0 and ISync_BV_Show_Stacked == 1 and itemCount and itemCount > 0 )then
						  costOfItem = floor(tonumber(costOfItem) * tonumber(itemCount));
					end
					
					--if there isn't a cost and or cost is = 0 and we don't want to show nill then
					--ignore
					if(not costOfItem or costOfItem < 1 and ISync_BV_Show_Nill == 1) then
						--do nothing cause we want to hide value for this item
					else
						
						if(not costOfItem or costOfItem < 1) then costOfItem = 0; end
					
	
						--get quality
						sParseLink = nil; --reset
						sParseLink = string.gsub(ISync:GetData(name, "info"), "^(.-):(.-):(.-):(.-):(.-):(.-):(.-):(.-):(.-):(.-)$", "%1");
					
	
						--insert into table
						ISync_BVList[iNew] = { };
						ISync_BVList[iNew].name = name;
						ISync_BVList[iNew].money = costOfItem;
						ISync_BVList[iNew].bag = bag;
						ISync_BVList[iNew].slot = slot;
						ISync_BVList[iNew].count = itemCount;
						ISync_BVList[iNew].quality = tonumber(sParseLink);
						ISync_BVList[iNew].texture = texture;
						ISync_BVList[iNew].sID = ISync:GetData(name, "item");

						iNew = iNew + 1;

					end
					

				end--if(itemCount) then

			end--if(name and name ~= "") then


		    end--if (link and link ~= "") then

		end--end for
	end--end for
    
    
    	ISync_BVList.onePastEnd = iNew;

	ISync:BV_Sort();
  
  
end


---------------------------------------------------
-- ISync:BV_Sort()
---------------------------------------------------
function ISync:BV_Sort()

	--only process if we have something to work with
	if not ISync_BVList then return; end
  
  		--sort by value of items
		if(ISync_BV_Sort_Value == 1) then

			table.sort(ISync_BVList, ISync_BV_SortByValue);
			
		--sort by rarity
		elseif(ISync_BV_Sort_Rarity == 1) then

			table.sort(ISync_BVList, ISync_BV_SortByRarity);
		
		--sort by name
		else
			table.sort(ISync_BVList, ISync_BV_SortByName);

		end

end


---------------------------------------------------
-- ISync_BV_SortByName
---------------------------------------------------
function ISync_BV_SortByName(a,b)

    return a.name < b.name;
    
end

---------------------------------------------------
-- ISync_BV_SortByValue
---------------------------------------------------
function ISync_BV_SortByValue(a,b)

	if(a.money and b.money) then
		return b.money < a.money;
	end

end

---------------------------------------------------
-- ISync_BV_SortByRarity
---------------------------------------------------
function ISync_BV_SortByRarity(elem1, elem2)

	local color1, color2;

	--get the corresponding quality
	color1 = elem1.quality;
	color2 = elem2.quality
	
	
	if(color1 and color2) then
	
		--this sorts the name if the colors match
		--that way the rarity are also alphabatized within the same rarity
		if( color1 == color2 ) then
			return elem1.name < elem2.name;
		end
	
		--return the color if it doesn't match
		return color2 < color1;
	
	else
		return nil;
	end
	
end


---------------------------------------------------
-- ISync:BV_UpdateScrollFrame()
---------------------------------------------------
function ISync:BV_UpdateScrollFrame()

	local iItem;

	--don't do if the frame isn't visible duh
	if (not ISync_BV_Frame:IsVisible()) then return; end
	
	--check to see if we have an index to work with, if not then build it
	if not ISync_BVList then ISync:BV_BuildIndex(); end --make sure

	--start the scroll process
	--15 items and height of 16 each
	FauxScrollFrame_Update(ISync_IC_ScrollFrame, ISync_BVList.onePastEnd - 1, 15, 16);

	--loop through the 15 frames
	for iItem = 1, 15, 1 do

		--grab the current item index and then get the itembar object itself
		local itemIndex = iItem + FauxScrollFrame_GetOffset(ISync_IC_ScrollFrame);
		local ISyncItemObj = getglobal("ISync_BrowseButton"..iItem);

		--if it's still within boundaries then procezz
		if( itemIndex < ISync_BVList.onePastEnd ) then

			--make sure the item is even in the array and that it has an sID
			if(ISync_BVList[itemIndex] and ISync_BVList[itemIndex].sID) then

				--store the id
				ISyncItemObj.storeID = ISync_BVList[itemIndex].sID;

				--give the itembar it's name
				if(ISync_BVList[itemIndex].name) then
					getglobal("ISync_BrowseButton"..iItem.."Name"):SetText(ISync_BVList[itemIndex].name);
				end

				--set color
				local color = { };
				if(ISync_BVList[itemIndex].quality) then
					color = ITEM_QUALITY_COLORS[tonumber(ISync_BVList[itemIndex].quality)];
					getglobal("ISync_BrowseButton"..iItem.."Name"):SetTextColor(color.r, color.g, color.b);
					getglobal("ISync_BrowseButton"..iItem.."Name").r = color.r;
					getglobal("ISync_BrowseButton"..iItem.."Name").g = color.g;
					getglobal("ISync_BrowseButton"..iItem.."Name").b = color.b;
				else
					getglobal("ISync_BrowseButton"..iItem.."Name").r = 0;
					getglobal("ISync_BrowseButton"..iItem.."Name").g = 0;
					getglobal("ISync_BrowseButton"..iItem.."Name").b = 0;
				end


				local moneyFrame = getglobal("ISync_BrowseButton"..iItem.."MoneyFrame");

				--set price if available
				if(ISync_BVList[itemIndex].money and moneyFrame) then

					MoneyFrame_Update(moneyFrame:GetName(), ISync_BVList[itemIndex].money);
					moneyFrame:Show();
				end


				--set texture
				if(ISync_BVList[itemIndex].texture) then
					getglobal("ISync_BrowseButton"..iItem.."ItemIconTexture"):SetTexture(ISync_BVList[itemIndex].texture);
				end

				--show stacked items
				if(ISync_BVList[itemIndex].count > 1 and ISync_BV_Show_Stacked == 1) then
					getglobal("ISync_BrowseButton"..iItem.."ItemCount"):SetText(ISync_BVList[itemIndex].count);
					getglobal("ISync_BrowseButton"..iItem.."ItemCount"):Show();
					
					--store the count
					ISyncItemObj.storeCount = ISync_BVList[itemIndex].count;

				else
					getglobal("ISync_BrowseButton"..iItem.."ItemCount"):Hide();
					ISyncItemObj.storeCount = 1; --default
				end
				
				
				--store the item array in both the item bar and texture
				ISyncItemObj.itemInfo = ISync_BVList[itemIndex];
				
				
				--show the item
				ISyncItemObj:Show(); 

			else
				--nothing to show then hide it
				ISyncItemObj:Hide();


			end

		else
			--error occured hide it
			ISyncItemObj:Hide();

		end

	end --end for
	

end



--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ISync:BV_ItemEnter()
---------------------------------------------------
function ISync:BV_ItemEnter()
local iItem;

	--get the item info
	if(this.itemInfo) then
		iItem = this.itemInfo;
	elseif(this:GetParent():GetName()) then
		iItem = getglobal(this:GetParent():GetName()).itemInfo;
	else
		iItem = nil;
	end
	
	
	if(this.storeID) then
	
		--check link
		local name_X, link_X, quality_X, minLevel_X, class_X, subclass_X, maxStack_X = GetItemInfo("item:"..this.storeID);

		if(name_X and link_X and ISync_RealmNum) then
		
		
			--show the sell cursor
			if(MerchantFrame:IsVisible() and not InRepairMode()) then

				ShowContainerSellCursor(iItem.bag, iItem.slot);

			elseif(MerchantFrame:IsVisible() and InRepairMode()) then

				--do nothing
			else
				ResetCursor();

			end

			ISync_BV_Frame.TooltipButton = this:GetID();
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink(link_X);

			--update the search information
			if(ISync.ParseTooltip and ISyncDB[ISync_RealmNum] and ISyncDB[ISync_RealmNum][name_X]) then
				UIParent.TooltipButton = 1;
				ISyncTooltip:SetOwner(UIParent, "ANCHOR_BOTTOMRIGHT");
				ISyncTooltip:SetHyperlink(link_X);
				ISync:ParseTooltip(name_X, ISync_RealmNum, this.storeID);
				ISyncTooltip:Hide();
			end
			
			--display prices in tooltips
			--the 1 at the end will force the tooltip to show
			if(ISync_AddTooltip and this.storeCount) then
				
				--update another mod if available
				if(ISync_CheckModTooltip) then
					ISync_CheckModTooltip(GameTooltip, name_X, "item:"..this.storeID, this.storeCount, quality_X);
					
				end
				ISync_AddTooltip(GameTooltip, name_X, this.storeID, quality_X, this.storeCount, nil, 1, 1);
			else
				--update another mod if available
				if(ISync_CheckModTooltip) then
					ISync_CheckModTooltip(GameTooltip, name_X, "item:"..this.storeID, 1, quality_X);
				end
				
				ISync_AddTooltip(GameTooltip, name_X, this.storeID, quality_X, 1, nil, 1, 1);
			end

		end
			
		
	end--end
	

end



---------------------------------------------------
-- ISync:BV_ItemClick()
---------------------------------------------------
function ISync:BV_ItemClick(sButton)
local iItem;

	--get the item info
	if(this.itemInfo) then
		iItem = this.itemInfo;
	elseif(this:GetParent():GetName()) then
		iItem = getglobal(this:GetParent():GetName()).itemInfo;
	else
		iItem = nil;
	end
	

	--make sure we have something to work with
	if(not iItem) then return; end
	
	
	--special thanks to Axu for the code :)  Support for AxuItemMenus
	if (AxuItemMenus_EvocationTest and AxuItemMenus_EvocationTest(sButton, "isync")) then
		
		AxuItemMenus_FillFromLink(GetContainerItemLink(iItem.bag, iItem.slot));
		AxuItemMenus_OpenMenu();
	
	elseif sButton == "LeftButton" then
		
		--check to see if shift it down to add the hyperlink
		if IsShiftKeyDown() then
		
		    if ChatFrameEditBox:IsVisible() then
		    
		    	--get the link
			local link = GetContainerItemLink(iItem.bag, iItem.slot);
			
			--if there is a link then add it
			if link then ChatFrameEditBox:Insert(link); end
			
		    end  -- editbox
		    
		elseif( IsControlKeyDown()) then
		

			--show dressing room
			DressUpItemLink(GetContainerItemLink(iItem.bag, iItem.slot));
		

		--if shift isn't pressed then they are trying to pickup the item
		else
		    PickupContainerItem(iItem.bag, iItem.slot);
		end  -- shift down
		

	elseif sButton == "RightButton" then
		
		UseContainerItem(iItem.bag, iItem.slot);
	end
	
	
	

end
